@@ -23,16 +23,13 @@ import subprocess
 import sys
 import time
 import unittest
+from collections import UserDict
 
 from UEFI.Filesystems import GrubShellBootableIsoImage
 from UEFI.SignedBinary import SignedBinary
 from UEFI.Qemu import QemuEfiMachine, QemuEfiVariant, QemuEfiFlashSize
 from UEFI import Qemu
 
-DPKG_ARCH = subprocess.check_output(
-    ['dpkg', '--print-architecture']
-).decode().rstrip()
-
 EfiArchToGrubArch = {
     'X64': "x86_64",
     'AA64': "arm64",
@@ -68,6 +65,17 @@ def get_local_shim_path(efi_arch, signed=False):
     return shim_path
 
 
+class DpkgArch(UserDict):
+    data = {}
+    dpkg_arch_out = subprocess.check_output(
+        ['dpkg-architecture'],
+    ).decode().rstrip().splitlines()
+
+    for var in dpkg_arch_out:
+        key, val = var.split('=')
+        data[key] = val
+
+
 class BootToShellTest(unittest.TestCase):
     debug = True
 
@@ -139,11 +147,19 @@ class BootToShellTest(unittest.TestCase):
             self.fail("%s\n" % (err))
         self.assertEqual(should_verify, verified)
 
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
     def test_aavmf(self):
         q = Qemu.QemuCommand(QemuEfiMachine.AAVMF)
         self.run_cmd_check_shell(q.command)
 
-    @unittest.skipUnless(DPKG_ARCH == 'arm64', "Requires grub-efi-arm64")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH'] == 'arm64', "Requires grub-efi-arm64",
+    )
     def test_aavmf_ms_secure_boot_signed(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.AAVMF,
@@ -155,7 +171,12 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'aa64', True)
 
-    @unittest.skipUnless(DPKG_ARCH == 'arm64', "Requires grub-efi-arm64")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH'] == 'arm64', "Requires grub-efi-arm64",
+    )
     def test_aavmf_ms_secure_boot_no_shell(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.AAVMF,
@@ -163,7 +184,12 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command, should_start=False)
 
-    @unittest.skipUnless(DPKG_ARCH == 'arm64', "Requires grub-efi-arm64")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH'] == 'arm64', "Requires grub-efi-arm64",
+    )
     def test_aavmf_ms_secure_boot_unsigned(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.AAVMF,
@@ -175,7 +201,10 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'aa64', False)
 
-    @unittest.skipUnless(DPKG_ARCH == 'arm64', "arm64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'arm64', "arm64-only")
     def test_aavmf_snakeoil_secure_boot_signed(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.AAVMF,
@@ -197,7 +226,10 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'aa64', True)
 
-    @unittest.skipUnless(DPKG_ARCH == 'arm64', "arm64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'arm64', "arm64-only")
     def test_aavmf_snakeoil_secure_boot_unsigned(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.AAVMF,
@@ -214,10 +246,16 @@ class BootToShellTest(unittest.TestCase):
         q = Qemu.QemuCommand(QemuEfiMachine.AAVMF32)
         self.run_cmd_check_shell(q.command)
 
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
     def test_loongarch64(self):
         q = Qemu.QemuCommand(QemuEfiMachine.LOONGARCH64)
         self.run_cmd_check_shell(q.command)
 
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
     def test_ovmf_4m_pc(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_PC,
@@ -225,6 +263,9 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command)
 
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
     def test_ovmf_4m_q35(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -232,7 +273,10 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_4m_ms_secure_boot_no_shell(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -241,7 +285,10 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command, should_start=False)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_4m_ms_secure_boot_signed(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -254,7 +301,10 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'x64', True)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_4m_ms_secure_boot_unsigned(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -267,7 +317,10 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'x64', False)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_snakeoil_secure_boot_no_shell(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -275,7 +328,10 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command, should_start=False)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_snakeoil_secure_boot_signed(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -297,7 +353,10 @@ class BootToShellTest(unittest.TestCase):
         q.add_disk(iso.path)
         self.run_cmd_check_secure_boot(q.command, 'x64', True)
 
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
+    @unittest.skipUnless(DpkgArch['DEB_BUILD_ARCH'] == 'amd64', "amd64-only")
     def test_ovmf_snakeoil_secure_boot_unsigned(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -324,6 +383,9 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command)
 
+    @unittest.skipUnless(
+        DpkgArch['DEB_BUILD_ARCH_BITS'] == '64', "Requires 64-bit QEMU",
+    )
     def test_riscv64(self):
         q = Qemu.QemuCommand(QemuEfiMachine.RISCV64)
         self.run_cmd_check_shell(q.command)
